# -*- coding: utf-8 -*-
__title__ = 'Universal Notification'
__version__ = '0.5.0'
__author__ = 'Pawel Krzyzaniak'
__license__ = 'Proprietary'
__copyright__ = 'Copyright 2016 Arabella'

# Version synonym
VERSION = __version__
